#include <jz/interrupt.h>
#include <jz/jz_intc.h>
#include <jz/jz_tcu.h>
#include <jz/jz_cpm.h>
#include <jz/jz_core.h>
#include <jz/ostimer.h>

#define JZ_TIMER_CH  2
#define JZ_TIMER_IRQ	IRQ_TCU2

/* memo:
   Jz475x
          IRQ_TCU0 OS Timer
          IRQ_TCU1 Chan 0,3,4,5 ( cannot work in sleep mode )
          IRQ_TCU2 Chan 1,2     ( can work in sleep mode )

   Jz474x
          IRQ_TCU0 Chan 0
          IRQ_TCU1 Chan 1
          IRQ_TCU2 Chan 2-7

end memo */

SIGNAL(JZ_TIMER_IRQ)
{
	__tcu_clear_full_match_flag(JZ_TIMER_CH);
	__intc_ack_irq(JZ_TIMER_IRQ);
}

int jz_timer_hz()
{
	int clk,d;
	const int div[] = {1, 4, 16, 64, 256, 1024, 1, 1};

	if (REG_TCU_TCSR(JZ_TIMER_TCU_CH) & TCU_OSTCSR_PCK_EN) {
		clk = __cpm_get_pclk();
	} else {
		clk = JZ_EXTAL;
	}
	d = ( REG_TCU_TCSR(JZ_TIMER_TCU_CH) & TCU_TCSR_PRESCALE_MASK) >> TCU_TCSR_PRESCALE_BIT;
	return clk/div[d];
}

void __jz_timer_start(uint32_t tcsr)
{
	REG_TCU_TCSR(JZ_TIMER_CH) = tcsr;
	REG_TCU_TCNT(JZ_TIMER_CH) = 0;

	REG_TCU_TMSR = ((1 << JZ_TIMER_CH) | (1 << (JZ_TIMER_CH + 16)));

	__tcu_counter_enabled(JZ_TIMER_CH);
	__tcu_clear_full_match_flag(JZ_TIMER_CH);
	__tcu_unmask_full_match_irq(JZ_TIMER_CH);
        __tcu_start_counter(JZ_TIMER_CH);
	__intc_unmask_irq(JZ_TIMER_IRQ);
}

void jz_timer_start(int is_fast)
{
	uint32_t tcsr;
	if (!is_fast) {
		/* ext clock (12MHz) / 4 = 3MHz : overflow 23.8 min */
		tcsr = TCU_TCSR_PRESCALE4 | TCU_TCSR_EXT_EN;
	} else {
		/* pclk (cpuclock / 1,2,3,4,6,8) 
		 * overflow 21.5 sec @ 200MHz 
		 * overflow 37.6 sec @ 114MHz */
		tcsr = TCU_TCSR_PRESCALE1 | TCU_TCSR_PCK_EN;
	}
	__jz_timer_start(tcsr);
}

void jz_timer_stop()
{
	__intc_mask_irq(JZ_TIMER_IRQ);
	__tcu_mask_full_match_irq(JZ_TIMER_CH);
	__tcu_stop_counter(JZ_TIMER_CH);

	REG_TCU_TCSR(JZ_TIMER_CH) = 0;
}
